/****************************************************************************
** Meta object code from reading C++ file 'preferencesdialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/preferencesdialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'preferencesdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17PreferencesDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto PreferencesDialog::qt_create_metaobjectdata<qt_meta_tag_ZN17PreferencesDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PreferencesDialog",
        "resultFormatUpdated",
        "",
        "resultDisplayUpdated",
        "expressionCalculationUpdated",
        "expressionFormatUpdated",
        "alwaysOnTopChanged",
        "enableTooltipsChanged",
        "titleTypeChanged",
        "resultFontChanged",
        "expressionFontChanged",
        "statusFontChanged",
        "keypadFontChanged",
        "appFontChanged",
        "symbolsUpdated",
        "historyExpressionTypeChanged",
        "binaryBitsChanged",
        "twosChanged",
        "statusModeChanged",
        "buttonLocationChanged",
        "automaticDigitGroupingChanged",
        "expressionPositionChanged",
        "showStatusBarChanged",
        "dialogClosed",
        "ignoreLocaleToggled",
        "keepAboveToggled",
        "tooltipsChanged",
        "preserveHeightChanged",
        "statusDelayChanged",
        "autocalcSelectionToggled",
        "calculateDelayChanged",
        "adaptiveDelayToggled",
        "binTwosToggled",
        "hexTwosToggled",
        "binTwosInputToggled",
        "hexTwosInputToggled",
        "bitsChanged",
        "lowerCaseToggled",
        "duodecimalSymbolsToggled",
        "multiplicationSignChanged",
        "divisionSlashToggled",
        "spellOutToggled",
        "eToggled",
        "imaginaryJToggled",
        "decimalCommaToggled",
        "ignoreCommaToggled",
        "ignoreDotToggled",
        "colorizeToggled",
        "formatToggled",
        "parsingModeChanged",
        "unknownsToggled",
        "temperatureCalculationChanged",
        "defaultCurrencyChanged",
        "exratesToggled",
        "exratesChanged",
        "unitsToggled",
        "binaryPrefixesToggled",
        "readPrecisionToggled",
        "simplifiedPercentageToggled",
        "intervalCalculationChanged",
        "complexFormChanged",
        "roundingChanged",
        "repeatingDecimalsChanged",
        "mixedUnitsToggled",
        "abbreviateNamesToggled",
        "conversionChanged",
        "prefixesChanged",
        "allPrefixesToggled",
        "denominatorPrefixToggled",
        "variableUnitsToggled",
        "updateCustomDigitGrouping",
        "groupingChanged",
        "cgfChanged",
        "cgsChanged",
        "intervalDisplayChanged",
        "conciseUncertaintyInputToggled",
        "limitImplicitToggled",
        "titleChanged",
        "resultFontClicked",
        "resultFontToggled",
        "expressionFontClicked",
        "expressionFontToggled",
        "statusFontClicked",
        "statusFontToggled",
        "keypadFontClicked",
        "keypadFontToggled",
        "appFontClicked",
        "appFontToggled",
        "darkModeToggled",
        "styleChanged",
        "langChanged",
        "factorizeToggled",
        "rpnKeysToggled",
        "replaceExpressionChanged",
        "autocopyResultToggled",
        "multipleInstancesToggled",
        "checkVersionToggled",
        "clearHistoryToggled",
        "maxHistoryLinesChanged",
        "historyExpressionChanged",
        "copyAsciiToggled",
        "copyAsciiWithoutUnitsToggled",
        "caretAsXorToggled",
        "automaticDigitGroupingToggled",
        "closeWithEscToggled",
        "disableCursorBlinkingToggled",
        "showPercentInNumpadToggled",
        "expressionPositionToggled",
        "showStatusBarToggled"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'resultFormatUpdated'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resultDisplayUpdated'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'expressionCalculationUpdated'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'expressionFormatUpdated'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'alwaysOnTopChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enableTooltipsChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'titleTypeChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resultFontChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'expressionFontChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'statusFontChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'keypadFontChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'appFontChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'symbolsUpdated'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'historyExpressionTypeChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'binaryBitsChanged'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'twosChanged'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'statusModeChanged'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'buttonLocationChanged'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'automaticDigitGroupingChanged'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'expressionPositionChanged'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showStatusBarChanged'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dialogClosed'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ignoreLocaleToggled'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'keepAboveToggled'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'tooltipsChanged'
        QtMocHelpers::SlotData<void(int)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'preserveHeightChanged'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'statusModeChanged'
        QtMocHelpers::SlotData<void(int)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'statusDelayChanged'
        QtMocHelpers::SlotData<void(int)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'autocalcSelectionToggled'
        QtMocHelpers::SlotData<void(bool)>(29, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'calculateDelayChanged'
        QtMocHelpers::SlotData<void(int)>(30, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'adaptiveDelayToggled'
        QtMocHelpers::SlotData<void(bool)>(31, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'binTwosToggled'
        QtMocHelpers::SlotData<void(bool)>(32, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'hexTwosToggled'
        QtMocHelpers::SlotData<void(bool)>(33, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'binTwosInputToggled'
        QtMocHelpers::SlotData<void(bool)>(34, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'hexTwosInputToggled'
        QtMocHelpers::SlotData<void(bool)>(35, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'bitsChanged'
        QtMocHelpers::SlotData<void(int)>(36, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'lowerCaseToggled'
        QtMocHelpers::SlotData<void(bool)>(37, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'duodecimalSymbolsToggled'
        QtMocHelpers::SlotData<void(bool)>(38, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'multiplicationSignChanged'
        QtMocHelpers::SlotData<void(int)>(39, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'divisionSlashToggled'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'spellOutToggled'
        QtMocHelpers::SlotData<void(bool)>(41, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'eToggled'
        QtMocHelpers::SlotData<void(bool)>(42, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'imaginaryJToggled'
        QtMocHelpers::SlotData<void(bool)>(43, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'decimalCommaToggled'
        QtMocHelpers::SlotData<void(bool)>(44, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'ignoreCommaToggled'
        QtMocHelpers::SlotData<void(bool)>(45, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'ignoreDotToggled'
        QtMocHelpers::SlotData<void(bool)>(46, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'colorizeToggled'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'formatToggled'
        QtMocHelpers::SlotData<void(bool)>(48, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'parsingModeChanged'
        QtMocHelpers::SlotData<void(int)>(49, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'unknownsToggled'
        QtMocHelpers::SlotData<void(bool)>(50, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'temperatureCalculationChanged'
        QtMocHelpers::SlotData<void(int)>(51, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'defaultCurrencyChanged'
        QtMocHelpers::SlotData<void(const QString &)>(52, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'exratesToggled'
        QtMocHelpers::SlotData<void(bool)>(53, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'exratesChanged'
        QtMocHelpers::SlotData<void(int)>(54, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'unitsToggled'
        QtMocHelpers::SlotData<void(bool)>(55, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'binaryPrefixesToggled'
        QtMocHelpers::SlotData<void(bool)>(56, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'readPrecisionToggled'
        QtMocHelpers::SlotData<void(bool)>(57, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'simplifiedPercentageToggled'
        QtMocHelpers::SlotData<void(bool)>(58, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'intervalCalculationChanged'
        QtMocHelpers::SlotData<void(int)>(59, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'complexFormChanged'
        QtMocHelpers::SlotData<void(int)>(60, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'roundingChanged'
        QtMocHelpers::SlotData<void(int)>(61, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'repeatingDecimalsChanged'
        QtMocHelpers::SlotData<void(int)>(62, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'mixedUnitsToggled'
        QtMocHelpers::SlotData<void(bool)>(63, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'abbreviateNamesToggled'
        QtMocHelpers::SlotData<void(bool)>(64, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'conversionChanged'
        QtMocHelpers::SlotData<void(int)>(65, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'prefixesChanged'
        QtMocHelpers::SlotData<void(int)>(66, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'allPrefixesToggled'
        QtMocHelpers::SlotData<void(bool)>(67, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'denominatorPrefixToggled'
        QtMocHelpers::SlotData<void(bool)>(68, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'variableUnitsToggled'
        QtMocHelpers::SlotData<void(bool)>(69, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'updateCustomDigitGrouping'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'groupingChanged'
        QtMocHelpers::SlotData<void(int)>(71, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'cgfChanged'
        QtMocHelpers::SlotData<void(int)>(72, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'cgsChanged'
        QtMocHelpers::SlotData<void(const QString &)>(73, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'intervalDisplayChanged'
        QtMocHelpers::SlotData<void(int)>(74, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'conciseUncertaintyInputToggled'
        QtMocHelpers::SlotData<void(bool)>(75, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'limitImplicitToggled'
        QtMocHelpers::SlotData<void(bool)>(76, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'titleChanged'
        QtMocHelpers::SlotData<void(int)>(77, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'resultFontClicked'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'resultFontToggled'
        QtMocHelpers::SlotData<void(bool)>(79, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'expressionFontClicked'
        QtMocHelpers::SlotData<void()>(80, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'expressionFontToggled'
        QtMocHelpers::SlotData<void(bool)>(81, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'statusFontClicked'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'statusFontToggled'
        QtMocHelpers::SlotData<void(bool)>(83, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'keypadFontClicked'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'keypadFontToggled'
        QtMocHelpers::SlotData<void(bool)>(85, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'appFontClicked'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'appFontToggled'
        QtMocHelpers::SlotData<void(bool)>(87, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'darkModeToggled'
        QtMocHelpers::SlotData<void(bool)>(88, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'styleChanged'
        QtMocHelpers::SlotData<void(int)>(89, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'langChanged'
        QtMocHelpers::SlotData<void(int)>(90, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'factorizeToggled'
        QtMocHelpers::SlotData<void(bool)>(91, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'rpnKeysToggled'
        QtMocHelpers::SlotData<void(bool)>(92, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'replaceExpressionChanged'
        QtMocHelpers::SlotData<void(int)>(93, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'autocopyResultToggled'
        QtMocHelpers::SlotData<void(bool)>(94, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'multipleInstancesToggled'
        QtMocHelpers::SlotData<void(bool)>(95, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'checkVersionToggled'
        QtMocHelpers::SlotData<void(bool)>(96, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'clearHistoryToggled'
        QtMocHelpers::SlotData<void(bool)>(97, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'maxHistoryLinesChanged'
        QtMocHelpers::SlotData<void(int)>(98, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'historyExpressionChanged'
        QtMocHelpers::SlotData<void(int)>(99, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'copyAsciiToggled'
        QtMocHelpers::SlotData<void(bool)>(100, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'copyAsciiWithoutUnitsToggled'
        QtMocHelpers::SlotData<void(bool)>(101, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'caretAsXorToggled'
        QtMocHelpers::SlotData<void(bool)>(102, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'automaticDigitGroupingToggled'
        QtMocHelpers::SlotData<void(bool)>(103, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'closeWithEscToggled'
        QtMocHelpers::SlotData<void(bool)>(104, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'disableCursorBlinkingToggled'
        QtMocHelpers::SlotData<void(bool)>(105, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'showPercentInNumpadToggled'
        QtMocHelpers::SlotData<void(bool)>(106, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'expressionPositionToggled'
        QtMocHelpers::SlotData<void(bool)>(107, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'showStatusBarToggled'
        QtMocHelpers::SlotData<void(bool)>(108, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PreferencesDialog, qt_meta_tag_ZN17PreferencesDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PreferencesDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PreferencesDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PreferencesDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17PreferencesDialogE_t>.metaTypes,
    nullptr
} };

void PreferencesDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PreferencesDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->resultFormatUpdated(); break;
        case 1: _t->resultDisplayUpdated(); break;
        case 2: _t->expressionCalculationUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->expressionFormatUpdated((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->alwaysOnTopChanged(); break;
        case 5: _t->enableTooltipsChanged(); break;
        case 6: _t->titleTypeChanged(); break;
        case 7: _t->resultFontChanged(); break;
        case 8: _t->expressionFontChanged(); break;
        case 9: _t->statusFontChanged(); break;
        case 10: _t->keypadFontChanged(); break;
        case 11: _t->appFontChanged(); break;
        case 12: _t->symbolsUpdated(); break;
        case 13: _t->historyExpressionTypeChanged(); break;
        case 14: _t->binaryBitsChanged(); break;
        case 15: _t->twosChanged(); break;
        case 16: _t->statusModeChanged(); break;
        case 17: _t->buttonLocationChanged(); break;
        case 18: _t->automaticDigitGroupingChanged(); break;
        case 19: _t->expressionPositionChanged(); break;
        case 20: _t->showStatusBarChanged(); break;
        case 21: _t->dialogClosed(); break;
        case 22: _t->ignoreLocaleToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->keepAboveToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->tooltipsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->preserveHeightChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->statusModeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 27: _t->statusDelayChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 28: _t->autocalcSelectionToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->calculateDelayChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 30: _t->adaptiveDelayToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->binTwosToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: _t->hexTwosToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->binTwosInputToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->hexTwosInputToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 35: _t->bitsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 36: _t->lowerCaseToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: _t->duodecimalSymbolsToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->multiplicationSignChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 39: _t->divisionSlashToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 40: _t->spellOutToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->eToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 42: _t->imaginaryJToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 43: _t->decimalCommaToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 44: _t->ignoreCommaToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 45: _t->ignoreDotToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 46: _t->colorizeToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 47: _t->formatToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 48: _t->parsingModeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 49: _t->unknownsToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 50: _t->temperatureCalculationChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 51: _t->defaultCurrencyChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 52: _t->exratesToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 53: _t->exratesChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 54: _t->unitsToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 55: _t->binaryPrefixesToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 56: _t->readPrecisionToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 57: _t->simplifiedPercentageToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 58: _t->intervalCalculationChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 59: _t->complexFormChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 60: _t->roundingChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 61: _t->repeatingDecimalsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 62: _t->mixedUnitsToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 63: _t->abbreviateNamesToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 64: _t->conversionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 65: _t->prefixesChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 66: _t->allPrefixesToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 67: _t->denominatorPrefixToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 68: _t->variableUnitsToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 69: _t->updateCustomDigitGrouping(); break;
        case 70: _t->groupingChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 71: _t->cgfChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 72: _t->cgsChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 73: _t->intervalDisplayChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 74: _t->conciseUncertaintyInputToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 75: _t->limitImplicitToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 76: _t->titleChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 77: _t->resultFontClicked(); break;
        case 78: _t->resultFontToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 79: _t->expressionFontClicked(); break;
        case 80: _t->expressionFontToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 81: _t->statusFontClicked(); break;
        case 82: _t->statusFontToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 83: _t->keypadFontClicked(); break;
        case 84: _t->keypadFontToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 85: _t->appFontClicked(); break;
        case 86: _t->appFontToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 87: _t->darkModeToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 88: _t->styleChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 89: _t->langChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 90: _t->factorizeToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 91: _t->rpnKeysToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 92: _t->replaceExpressionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 93: _t->autocopyResultToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 94: _t->multipleInstancesToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 95: _t->checkVersionToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 96: _t->clearHistoryToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 97: _t->maxHistoryLinesChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 98: _t->historyExpressionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 99: _t->copyAsciiToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 100: _t->copyAsciiWithoutUnitsToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 101: _t->caretAsXorToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 102: _t->automaticDigitGroupingToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 103: _t->closeWithEscToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 104: _t->disableCursorBlinkingToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 105: _t->showPercentInNumpadToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 106: _t->expressionPositionToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 107: _t->showStatusBarToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::resultFormatUpdated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::resultDisplayUpdated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)(int )>(_a, &PreferencesDialog::expressionCalculationUpdated, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)(bool )>(_a, &PreferencesDialog::expressionFormatUpdated, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::alwaysOnTopChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::enableTooltipsChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::titleTypeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::resultFontChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::expressionFontChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::statusFontChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::keypadFontChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::appFontChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::symbolsUpdated, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::historyExpressionTypeChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::binaryBitsChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::twosChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::statusModeChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::buttonLocationChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::automaticDigitGroupingChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::expressionPositionChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::showStatusBarChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (PreferencesDialog::*)()>(_a, &PreferencesDialog::dialogClosed, 21))
            return;
    }
}

const QMetaObject *PreferencesDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PreferencesDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PreferencesDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int PreferencesDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 108)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 108;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 108)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 108;
    }
    return _id;
}

// SIGNAL 0
void PreferencesDialog::resultFormatUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void PreferencesDialog::resultDisplayUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void PreferencesDialog::expressionCalculationUpdated(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void PreferencesDialog::expressionFormatUpdated(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void PreferencesDialog::alwaysOnTopChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void PreferencesDialog::enableTooltipsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void PreferencesDialog::titleTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void PreferencesDialog::resultFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void PreferencesDialog::expressionFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void PreferencesDialog::statusFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void PreferencesDialog::keypadFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void PreferencesDialog::appFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void PreferencesDialog::symbolsUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void PreferencesDialog::historyExpressionTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void PreferencesDialog::binaryBitsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void PreferencesDialog::twosChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void PreferencesDialog::statusModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void PreferencesDialog::buttonLocationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void PreferencesDialog::automaticDigitGroupingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void PreferencesDialog::expressionPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void PreferencesDialog::showStatusBarChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void PreferencesDialog::dialogClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}
QT_WARNING_POP
