/****************************************************************************
** Meta object code from reading C++ file 'unitsdialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/unitsdialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'unitsdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11UnitsDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto UnitsDialog::qt_create_metaobjectdata<qt_meta_tag_ZN11UnitsDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "UnitsDialog",
        "itemsChanged",
        "",
        "favouritesChanged",
        "variableRemoved",
        "Variable*",
        "variableDeactivated",
        "insertUnitRequest",
        "Unit*",
        "convertToUnitRequest",
        "unitActivated",
        "selectedCategoryChanged",
        "QTreeWidgetItem*",
        "selectedUnitChanged",
        "QModelIndex",
        "newClicked",
        "editClicked",
        "delClicked",
        "insertClicked",
        "convertClicked",
        "deactivateClicked",
        "searchChanged",
        "fromChanged",
        "toChanged",
        "toUnitChanged",
        "fromUnitChanged",
        "favouriteClicked",
        "showHiddenToggled",
        "onUnitActivated",
        "reject"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'itemsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'favouritesChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'variableRemoved'
        QtMocHelpers::SignalData<void(Variable *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'variableDeactivated'
        QtMocHelpers::SignalData<void(Variable *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'insertUnitRequest'
        QtMocHelpers::SignalData<void(Unit *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Signal 'convertToUnitRequest'
        QtMocHelpers::SignalData<void(Unit *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Signal 'unitActivated'
        QtMocHelpers::SignalData<void(Unit *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Slot 'selectedCategoryChanged'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, QTreeWidgetItem *)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 2 }, { 0x80000000 | 12, 2 },
        }}),
        // Slot 'selectedUnitChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 2 }, { 0x80000000 | 14, 2 },
        }}),
        // Slot 'newClicked'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editClicked'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'delClicked'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'insertClicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'convertClicked'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'deactivateClicked'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'searchChanged'
        QtMocHelpers::SlotData<void(const QString &)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'fromChanged'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'toChanged'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'toUnitChanged'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'fromUnitChanged'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'favouriteClicked'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'showHiddenToggled'
        QtMocHelpers::SlotData<void(bool)>(27, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'onUnitActivated'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Slot 'reject'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UnitsDialog, qt_meta_tag_ZN11UnitsDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject UnitsDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11UnitsDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11UnitsDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11UnitsDialogE_t>.metaTypes,
    nullptr
} };

void UnitsDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UnitsDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemsChanged(); break;
        case 1: _t->favouritesChanged(); break;
        case 2: _t->variableRemoved((*reinterpret_cast<std::add_pointer_t<Variable*>>(_a[1]))); break;
        case 3: _t->variableDeactivated((*reinterpret_cast<std::add_pointer_t<Variable*>>(_a[1]))); break;
        case 4: _t->insertUnitRequest((*reinterpret_cast<std::add_pointer_t<Unit*>>(_a[1]))); break;
        case 5: _t->convertToUnitRequest((*reinterpret_cast<std::add_pointer_t<Unit*>>(_a[1]))); break;
        case 6: _t->unitActivated((*reinterpret_cast<std::add_pointer_t<Unit*>>(_a[1]))); break;
        case 7: _t->selectedCategoryChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[2]))); break;
        case 8: _t->selectedUnitChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 9: _t->newClicked(); break;
        case 10: _t->editClicked(); break;
        case 11: _t->delClicked(); break;
        case 12: _t->insertClicked(); break;
        case 13: _t->convertClicked(); break;
        case 14: _t->deactivateClicked(); break;
        case 15: _t->searchChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->fromChanged(); break;
        case 17: _t->toChanged(); break;
        case 18: _t->toUnitChanged(); break;
        case 19: _t->fromUnitChanged(); break;
        case 20: _t->favouriteClicked(); break;
        case 21: _t->showHiddenToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->onUnitActivated((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 23: _t->reject(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UnitsDialog::*)()>(_a, &UnitsDialog::itemsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnitsDialog::*)()>(_a, &UnitsDialog::favouritesChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnitsDialog::*)(Variable * )>(_a, &UnitsDialog::variableRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnitsDialog::*)(Variable * )>(_a, &UnitsDialog::variableDeactivated, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnitsDialog::*)(Unit * )>(_a, &UnitsDialog::insertUnitRequest, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnitsDialog::*)(Unit * )>(_a, &UnitsDialog::convertToUnitRequest, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (UnitsDialog::*)(Unit * )>(_a, &UnitsDialog::unitActivated, 6))
            return;
    }
}

const QMetaObject *UnitsDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UnitsDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11UnitsDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int UnitsDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 24;
    }
    return _id;
}

// SIGNAL 0
void UnitsDialog::itemsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void UnitsDialog::favouritesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void UnitsDialog::variableRemoved(Variable * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void UnitsDialog::variableDeactivated(Variable * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void UnitsDialog::insertUnitRequest(Unit * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void UnitsDialog::convertToUnitRequest(Unit * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void UnitsDialog::unitActivated(Unit * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
